Package: bspm
Type: Package
Title: Bridge to System Package Manager
Version: 0.5.8
Authors@R: c(
    person("Iñaki", "Ucar", email="iucar@fedoraproject.org",
      role=c("aut", "cph", "cre"), comment=c(ORCID="0000-0001-6403-5550")))
Description: Enables binary package installations on Linux distributions.
    Provides functions to manage packages via the distribution's package
    manager. Also provides transparent integration with R's install.packages()
    and a fallback mechanism. When installed as a system package, interacts
    with the system's package manager without requiring administrative
    privileges via an integrated D-Bus service; otherwise, uses sudo.
    Currently, the following backends are supported: DNF, APT, ALPM.
License: MIT + file LICENSE
Encoding: UTF-8
OS_type: unix
SystemRequirements: systemd, dbus-python, PyGObject,
        python-(dnf|apt|alpm)
Suggests: tinytest
URL: https://cran4linux.github.io/bspm/
BugReports: https://github.com/cran4linux/bspm/issues
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-29 15:16:34 UTC; iucar
Author: Iñaki Ucar [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0001-6403-5550>)
Maintainer: Iñaki Ucar <iucar@fedoraproject.org>
Repository: CRAN
Date/Publication: 2026-01-29 16:50:08 UTC
Built: R 4.6.0; ; 2026-04-25 17:16:13 UTC; unix
